@@ -29,15 +29,52 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <stdint.h>
 
+#include <config.h>
 #include <mqtt3.h>
 
 int mqtt3_raw_connack(mqtt3_context *context, uint8_t result)
 {
-	if(mqtt3_write_byte(context, CONNACK)) return 1;
-	if(mqtt3_write_remaining_length(context, 2)) return 1;
-	if(mqtt3_write_byte(context, 0)) return 1;
-	if(mqtt3_write_byte(context, result)) return 1;
+	struct _mqtt3_packet *packet = NULL;
 
+	if(context) mqtt3_log_printf(MQTT3_LOG_DEBUG, "Sending CONNACK to %s (%d)", context->id, result);
+
+	packet = mqtt3_calloc(1, sizeof(struct _mqtt3_packet));
+	if(!packet) return 1;
+
+	packet->command = CONNACK;
+	packet->remaining_length = 2;
+	packet->payload = mqtt3_malloc(sizeof(uint8_t)*2);
+	if(!packet->payload){
+		mqtt3_free(packet);
+		return 1;
+	}
+	packet->payload[0] = 0;
+	packet->payload[1] = result;
+
+	if(mqtt3_net_packet_queue(context, packet)) return 1;
 	return 0;
 }
 
+int mqtt3_raw_suback(mqtt3_context *context, uint16_t mid, uint32_t payloadlen, const uint8_t *payload)
+{
+	struct _mqtt3_packet *packet = NULL;
+
+	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Sending SUBACK to %s", context->id);
+
+	packet = mqtt3_calloc(1, sizeof(struct _mqtt3_packet));
+	if(!packet) return 1;
+
+	packet->command = SUBACK;
+	packet->remaining_length = 2+payloadlen;
+	packet->payload = mqtt3_malloc(sizeof(uint8_t)*(2+payloadlen));
+	if(!packet->payload){
+		mqtt3_free(packet);
+		return 1;
+	}
+	if(mqtt3_write_uint16(packet, mid)) return 1;
+	if(mqtt3_write_bytes(packet, payload, payloadlen)) return 1;
+
+	if(mqtt3_net_packet_queue(context, packet)) return 1;
+
+	return 0;
+}
